from math import ceil

from util.quantor import forall

def taskset_is_suspension_free(taskset):
    "compatibility: use when blocking and suspensions are not an issue"
    for t in taskset:
        t.blocked = 0
        t.suspended = 0

def fp_demand(task, time):
    return task.cost * int(ceil(float(time) / task.period))

def tda_schedulable(taskset, i):
    task = taskset[i]
    higher_prio = taskset[:i]

    test_end = task.deadline


    # pre-compute the additional terms for the processor demand bound
    added_demand = task.blocked + task.cost

    # see if we find a point where the demand is satisfied
    time = sum([t.cost for t in higher_prio]) + added_demand
    while time <= test_end:
        # Account for higher-priority interference due to double-hit /
        # back-to-back execution with the formula given in LNR:09.
        demand = sum([fp_demand(t, time + t.suspended) for t in higher_prio]) \
            + added_demand
        if demand <= time:
            # yep, demand will be met by time
            return True
        else:
            # try again
            time = demand

    # if we get here, we didn't converge
    return False

def is_schedulable(taskset):
    assert forall(taskset)(lambda t: t.deadline <= t.period)
    for i, t in enumerate(taskset):
        if not tda_schedulable(taskset, i):
            return False
    return True
