
import unittest
import model.resources as m

class GroupingAndNesting(unittest.TestCase):

    def setUp(self):
        id = 10
        self.t1 = 'T1'
        self.t2 = 'T2'
        self.x = m.Resource(id + 1)
        self.y = m.Resource(id + 2)
        self.z = m.Resource(id + 3)
        self.a = self.x.Request(self.t1, 10)
        self.b = m.Request(self.t2, self.y, 11)
        self.c = self.b.Nested(self.z, 100)

    def testGrouping(self):
        self.assertEqual(self.z.group,
                         self.y.group)

    def testDuration(self):
        self.assertEqual(self.b.duration,
                         self.b.raw_duration + self.c.duration)

    def testTasks(self):
        self.assertEqual(self.x.group.tasks,
                         frozenset([self.t1]))
        self.assertEqual(self.y.group.tasks,
                         frozenset([self.t2]))

    def testTaskReeval(self):
        self.assertEqual(self.y.group.tasks,
                         frozenset([self.t2]))
        self.d = m.Request(self.t1, self.y, 29)
        self.assertEqual(self.y.group.tasks,
                         frozenset([self.t2, self.t1]))
        self.assertNotEqual(self.y.group,
                            self.x.group)

    def testTaskReevalRegroup(self):
        self.assertEqual(self.y.group.tasks,
                         frozenset([self.t2]))
        self.d = self.a.Nested(self.y, 29)
        self.assertEqual(self.y.group.tasks,
                         frozenset([self.t2, self.t1]))
        self.assertEqual(self.y.group,
                         self.x.group)

    def testRequestsReeval(self):
        self.assertEqual(self.y.group.requests,
                         [self.b, self.c])
        self.d = m.Request(self.t1, self.y, 29)
        self.assertEqual(self.y.group.requests,
                         [self.b, self.c, self.d])
        self.assertNotEqual(self.y.group,
                            self.x.group)

    def testRequestsReevalRegroup(self):
        self.assertEqual(self.y.group.requests,
                         [self.b, self.c])
        self.d = self.a.Nested(self.y, 29)
        self.assertEqual(self.y.group.requests,
                         [self.b, self.c, self.a, self.d])
        self.assertEqual(self.y.group,
                         self.x.group)


if __name__ == '__main__':
    unittest.main()

