from util    import is_multiple, ordered_pairs
from util.quantor import forall
from math    import floor, ceil

# dyn prio slides, slide 15
def edf_schedulable(ts):
    return ts.density() <= 1.0

def np_edf_blocking_term(ts, i, L):
    return ceil(ts[i].cost) + sum([floor((L - 1) / ts[j].period) * ceil(ts[j].cost)
                             for j in xrange(0, i)])

def np_edf_blocking_ok(ts):
     return forall(
         xrange(0, len(ts))
         )(
         lambda i: forall(
             xrange(ts[0].period, ts[i].period)
             )(
             lambda L: L >= np_edf_blocking_term(ts,i, L)
             )
         )

def np_edf_schedulable(ts):
    assert forall(
        ordered_pairs(ts)
        )(
        lambda (t1, t2): t1.period <= t2.period
        )
    return edf_schedulable(ts) and np_edf_blocking_ok(ts)

def rm_schedulable(ts):
    assert forall(ts)(lambda t: t.period == t.deadline)
    u = ts.utilization()
    return (is_simply_periodic(ts) and u <= 1.0) or u <= rm_bound(len(ts))

def rm_bound(n):
    return n * (2.0 ** (1.0/n) - 1)

def is_simply_periodic(ts):
    return forall(
        ordered_pairs(ts)
        )(
        lambda (t1, t2): is_multiple(t1.period, t2.period)
        )

# static prio, slide 46
def tda_demand(ts, i):
    def demand(t):
        return ts[i].cost + (
               sum([ceil(float(t)/tsk.period) * tsk.cost
                   for tsk in ts[0:i]]))
    return demand

def tda_schedulable_task(ts, i, explain):
    demand = tda_demand(ts, i)
    t = sum([tsk.cost for tsk in ts[0:i+1]])
    d = demand(t)
    while abs(t - d) > 0.00001 and t <= ts[i].deadline:
        t = d
        d = demand(t)
    if explain:
        if t < ts[i].deadline:
            print "demand of T%02d fulfilled at %f." % (ts[i].id, t)
        else:
            print "demand of T%02d cannot be fulfilled by deadline" % ts[i].id
    return t <= ts[i].deadline

def tda_schedulable(ts, explain=False):
    assert forall(ts)(lambda t: t.deadline <= t.period)
    return forall(
        xrange(0, len(ts))
        )(
        lambda i: tda_schedulable_task(ts, i, explain)
        )
