# various time-related helpers

from datetime     import datetime, timedelta

import resource


def us2ms(us):
    """You have: microseconds
       You want: milliseconds
               * 0.001
               / 1000
    """
    return us * 0.001

def ms2us(ms):
    return ms * 1000

def sec2us(sec):
    return ms2us(sec * 1000)


def get_exec_time():
    usage = resource.getrusage(resource.RUSAGE_SELF)
    return usage.ru_utime + usage.ru_stime


class Timer(object):
    def __init__(self, delegate=None):
        self.elapsed = timedelta(0, 0, 0)
        self.delegate = delegate

    def __call__(self, *args, **kargs):
        self.start()
        x = self.delegate(*args, **kargs)
        self.stop()
        return x

    def start(self):
        self.start_time = datetime.now()
        return self.start_time

    def stop(self):
        stop = datetime.now()
        self.elapsed += stop - self.start_time
        del self.start_time
        return stop

    def seconds(self):
        secs  = self.elapsed.days * 3600.0 * 24.0
        secs += self.elapsed.seconds
        secs += self.elapsed.microseconds / 1000000.0
        return secs


class ExecTimer(object):
    def __init__(self, delegate=None):
        self.elapsed  = 0.0
        self.delegate = delegate
        self.count    = 0
        self.max      = 0.0
        self.digits   = 6

    def __call__(self, *args, **kargs):
        self.start()
        x = self.delegate(*args, **kargs)
        self.stop()
        return x

    def start(self):
        self.start_time = get_exec_time()
        return self.start_time

    def stop(self):
        stop  = get_exec_time()
        delta = stop - self.start_time

        self.elapsed += delta
        self.max = max(delta, self.max)
        self.count += 1

        del self.start_time
        return stop

    def seconds(self):
        return self.elapsed

    def seconds_avg(self):
        return round(self.elapsed / (self.count if self.count else 1), self.digits)

    def seconds_max(self):
        return round(self.max, self.digits)

